@@ -3,12 +3,14 @@
  * src/properties.c - control panel applet for gnome-volume-manager
  *
  * Robert Love <rml@novell.com>
+ * Jeffrey Stedfast <fejj@novell.com>
  *
- * (C) Copyright 2005 Novell, Inc.
+ * (C) Copyright 2005-2007 Novell, Inc.
  *
  * Licensed under the GNU GPL v2.  See COPYING.
  */
 
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -67,6 +69,8 @@ enum {
 	AUTOTABLET_COMMAND,
 	AUTOVIDEOCAM,
 	AUTOVIDEOCAM_COMMAND,
+	AUTOWEBCAM,
+	AUTOWEBCAM_COMMAND,
 	AUTORUN_PATH,
 	AUTOOPEN_PATH,
 };
@@ -109,7 +113,9 @@ static struct {
 	{ GCONF_ROOT "autotablet_command",        TYPE_STRING, NULL, FALSE },
 	{ GCONF_ROOT "autovideocam",              TYPE_BOOL,   NULL, TRUE  },
 	{ GCONF_ROOT "autovideocam_command",      TYPE_STRING, NULL, FALSE },
-	/* The following entries do not yet have a UI */
+	{ GCONF_ROOT "autowebcam",                TYPE_BOOL,   NULL, TRUE  },
+	{ GCONF_ROOT "autowebcam_command",        TYPE_STRING, NULL, FALSE },
+	/* The following entries do not (yet) have a UI */
 	/*{ GCONF_ROOT "autorun_path",         TYPE_STRING, NULL, FALSE },*/
 	/*{ GCONF_ROOT "autoopen_path",        TYPE_STRING, NULL, FALSE },*/
 };
@@ -159,10 +165,10 @@ toggled_cb (GtkToggleButton *toggle, const char *key)
 }
 
 static void
-gconf_changed_cb (GConfClient *client __attribute__((__unused__)),
-		  guint id __attribute__((__unused__)),
+gconf_changed_cb (GConfClient *client GNUC_UNUSED,
+		  guint id GNUC_UNUSED,
 		  GConfEntry *entry,
-		  gpointer data __attribute__((__unused__)))
+		  gpointer data GNUC_UNUSED)
 {
 	GConfValue *value;
 	gpointer result;
@@ -226,8 +232,8 @@ set_sensitivity (void)
  */
 static void
 close_cb (GtkWidget *dialog,
-	  int response __attribute__((__unused__)),
-	  gpointer user_data __attribute__((__unused__)))
+	  int response GNUC_UNUSED,
+	  gpointer user_data GNUC_UNUSED)
 {
 	gtk_widget_destroy (dialog);
 	gtk_main_quit ();
@@ -238,46 +244,54 @@ close_cb (GtkWidget *dialog,
  * a given GtkImage.
  */
 static void
-set_icon (GtkImage *image, GtkIconTheme *theme, const char *name)
+set_icon (GtkImage *image, GtkIconTheme *theme, const char *name, const char *fallback)
 {
 	GdkPixbuf *pixbuf;
 	struct stat st;
 	
+retry:
+	
 	if (*name == '/') {
 		if (stat (name, &st) == 0 && S_ISREG (st.st_mode)) {
 			gtk_image_set_from_file (image, name);
 			return;
-		} else
-			name = "gnome-dev-broken-image";
+		} else if (fallback && name != fallback) {
+			name = fallback;
+			goto retry;
+		} else {
+			name = "image-missing";
+		}
 	}
 	
 	if ((pixbuf = gtk_icon_theme_load_icon (theme, name, 48, 0, NULL))) {
 		gtk_image_set_from_pixbuf (image, pixbuf);
 		g_object_unref (pixbuf);
+	} else if (fallback && name != fallback) {
+		name = fallback;
+		goto retry;
 	}
 }
 
 static struct {
 	const char *name;
 	const char *icon;
+	const char *fallback;
 } icons[] = {
-	{ "audio_cd_image",         "gnome-dev-cdrom-audio"   },
-	{ "blank_cd_image",         "gnome-dev-disc-cdr"      },
-	{ "digital_camera_image",   "applets-screenshooter"   },
-	{ "dvd_video_image",        "gnome-dev-dvd"           },
-	{ "ipod_image",             "gnome-dev-ipod"          },
-	{ "keyboard_image",         "gnome-dev-keyboard"      },
-	{ "mouse_image",            "gnome-dev-mouse-optical" },
-	{ "palm_image",             "palm-pilot"              },
-	/* FIXME: need a new icon */
-	{ "pocketpc_image",         "palm-pilot"              },
-	{ "printer_image",          "gnome-dev-printer"       },
-	{ "removable_drives_image", "gnome-dev-removable"     },
-	{ "scanner_image",          "xsane"                   },
-	/* FIXME: need better icon */
-	{ "tablet_image",           ICONSDIR "/gvm-dev-tablet.png" },
-	/* FIXME: need better icon */
-	{ "videocam_image",         "gnome-mime-video"        }
+	{ "audio_cd_image",         "gnome-dev-cdrom-audio", "media-optical"  },
+	{ "blank_cd_image",         "gnome-dev-disc-cdr",    "media-optical"  },
+	{ "digital_camera_image",   "camera-photo",          NULL             },
+	{ "dvd_video_image",        "gnome-dev-dvd",         "media-optical"  },
+	{ "ipod_image",             "multimedia-player",     "gnome-dev-ipod" },
+	{ "keyboard_image",         "input-keyboard",        NULL             },
+	{ "mouse_image",            "input-mouse",           NULL             },
+	{ "palm_image",             "pda-palm",              "palm-pilot"     },
+	{ "pocketpc_image",         "pda-pocketpc",          "palm-pilot"     },
+	{ "printer_image",          "printer",               NULL             },
+	{ "removable_drives_image", "drive-removable-media", NULL             },
+	{ "scanner_image",          "scanner",               NULL             },
+	{ "tablet_image",           "input-tablet",          NULL             },
+	{ "videocam_image",         "camera-video",          NULL             },
+	{ "webcam_image",           "camera-web",            NULL             }
 };
 
 static void
@@ -288,7 +302,7 @@ theme_changed_cb (GtkIconTheme *theme, GladeXML *xml)
 	
 	for (i = 0; i < G_N_ELEMENTS (icons); i++) {
 		icon = glade_xml_get_widget (xml, icons[i].name);
-		set_icon (GTK_IMAGE (icon), theme, icons[i].icon);
+		set_icon (GTK_IMAGE (icon), theme, icons[i].icon, icons[i].fallback);
 	}
 }
 
@@ -459,7 +473,7 @@ check_daemon (void)
 	if (!need_daemon || check_clipboard ()) 
 		return;
 	
-	argv[0] = BINDIR "/gnome-volume-manager";
+	argv[0] = LIBEXECDIR "/gnome-volume-manager";
 	argv[1] = NULL;
 	g_spawn_async (g_get_home_dir (), argv, NULL, 0, NULL,
 			NULL, NULL, &error);
@@ -503,9 +517,7 @@ main (int argc, char **argv)
 		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
 							  _("The \"hald\" service is required but not currently "
 							    "running. Enable the service and rerun this application, "
-							    "or contact your system administrator.\n\n"
-							    "Note: You need Linux kernel 2.6 for volume "
-							    "management to work."));
+							    "or contact your system administrator."));
 		
 		gtk_dialog_run (GTK_DIALOG (dialog));
 		gtk_widget_destroy (dialog);
